# Sashite::CGN

This module provides a Ruby interface for data serialization in [CGN](http://sashite.wiki/General_Gameplay_Notation) format.

## Status

* [![Gem Version](https://badge.fury.io/rb/sashite-cgn.svg)](//badge.fury.io/rb/sashite-cgn)
* [![Build Status](https://secure.travis-ci.org/sashite/cgn.rb.svg?branch=master)](//travis-ci.org/sashite/cgn.rb?branch=master)
* [![Dependency Status](https://gemnasium.com/sashite/cgn.rb.svg)](//gemnasium.com/sashite/cgn.rb)

## Installation

Add this line to your application's Gemfile:

    gem 'sashite-cgn'

And then execute:

    $ bundle

Or install it yourself as:

    $ gem install sashite-cgn

## Usage

Working with CGN can be very simple, for example:

```ruby
require 'sashite-cgn'

# Parse a CGN string
Sashite::CGN::Ability.load("t<self>_&_^shift[-1,0]_=_@f+all~_@f+all%self") # =>
{
  subject: {
    :"...ally?" => true,
    :"actor" => :self,
    :"state" => {
      :"...last_moved_actor?" => nil,
      :"...previous_moves_counter" => nil
    }
  },

  verb: {
    name: :shift,
    vector: {direction: [-1,0], :"...maximum_magnitude" => nil}
  },

  object: {
    src_square: {
      :"...attacked?" => nil,
      :"...occupied!" => false,
      area: :all
    },
    dst_square: {
      :"...attacked?" => nil,
      :"...occupied!" => false,
      area: :all
    },
    promotable_into_actors: [:self]
  }
}

# Emit some CGN
Sashite::CGN::Subject.dump({
  :"...ally?" => true,
  :"actor"    => :self,
  :"state"    => {
    :"...last_moved_actor?" => nil,
    :"...previous_moves_counter" => nil
  }
}) # => "t<self>_&_"

## Contributing

1. Fork it
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create a new Pull Request
