require_relative '_test_helper'

describe Sashite::CGN::Ability do
  subject { Sashite::CGN::Ability }

  let(:cgn_document) { 't<self>_&_^shift[-1,0]_=_@f+all~_@f+all%self' }
  let(:ruby_object) do
    {
      subject: {
        :"...ally?" => true,
        :"actor" => :self,
        :"state" => {
          :"...last_moved_actor?" => nil,
          :"...previous_moves_counter" => nil
        }
      },

      verb: {
        name: :shift,
        vector: {direction: [-1,0], :"...maximum_magnitude" => nil}
      },

      object: {
        src_square: {
          :"...attacked?" => nil,
          :"...occupied!" => false,
          area: :all
        },
        dst_square: {
          :"...attacked?" => nil,
          :"...occupied!" => false,
          area: :all
        },
        promotable_into_actors: [:self]
      }
    }
  end

  describe '.load' do
    it 'parses a CGN string' do
      subject.load(cgn_document).hash.must_equal ruby_object.hash
    end

    describe 'errors' do
      it 'raises without an ability' do
        -> { subject.load 'foobar' }.must_raise ArgumentError
      end
    end
  end

  describe '.dump' do
    it 'emits some CGN' do
      subject.dump(ruby_object).must_equal cgn_document
    end

    describe 'errors' do
      it 'raises without an ability' do
        -> { subject.dump 'foobar' }.must_raise ArgumentError
      end
    end
  end
end
