require_relative '_test_helper'

describe Sashite::CGN::MaximumMagnitude do
  subject { Sashite::CGN::MaximumMagnitude }

  describe '.load' do
    describe 'null' do
      let(:cgn_document) { '_' }
      let(:ruby_object) { nil }

      it 'parses a CGN string' do
        subject.load(cgn_document).must_equal ruby_object
      end

      describe 'errors' do
        it 'raises an error' do
          -> { subject.load 'foobar' }.must_raise ArgumentError
        end
      end
    end

    describe 'an unsigned integer' do
      let(:cgn_document) { '42' }
      let(:ruby_object) { 42 }

      it 'parses a CGN string' do
        subject.load(cgn_document).must_equal ruby_object
      end

      describe 'errors' do
        it 'raises an error' do
          -> { subject.load '-4' }.must_raise ArgumentError
        end
      end
    end
  end

  describe '.dump' do
    describe 'null' do
      let(:cgn_document) { '_' }
      let(:ruby_object) { nil }

      it 'emits some CGN' do
        subject.dump(ruby_object).must_equal cgn_document
      end

      describe 'errors' do
        it 'raises an error' do
          -> { subject.dump 'foobar' }.must_raise ArgumentError
        end
      end
    end

    describe 'an unsigned integer' do
      let(:cgn_document) { '42' }
      let(:ruby_object) { 42 }

      it 'emits some CGN' do
        subject.dump(ruby_object).must_equal cgn_document
      end

      describe 'errors' do
        it 'raises an error' do
          -> { subject.dump '-4' }.must_raise ArgumentError
        end
      end
    end
  end
end
