require_relative '_test_helper'

describe Sashite::CGN::Occupied do
  subject { Sashite::CGN::Occupied }

  describe '.load' do
    describe 'a relationship' do
      let(:cgn_document) { 'an_ally_actor' }
      let(:ruby_object) { :an_ally_actor }

      it 'parses a CGN string' do
        subject.load(cgn_document).must_equal ruby_object
      end

      describe 'errors' do
        it 'raises without a relationship' do
          -> { subject.load 'foobar' }.must_raise ArgumentError
        end
      end
    end

    describe 'a boolean' do
      let(:cgn_document) { 'f' }
      let(:ruby_object) { false }

      it 'parses a CGN string' do
        subject.load(cgn_document).must_equal ruby_object
      end

      describe 'errors' do
        it 'raises without a boolean' do
          -> { subject.load 'foobar' }.must_raise ArgumentError
        end
      end
    end

    describe 'null' do
      let(:cgn_document) { '_' }
      let(:ruby_object) { nil }

      it 'parses a CGN string' do
        subject.load(cgn_document).must_equal ruby_object
      end

      describe 'errors' do
        it 'raises without null' do
          -> { subject.load '' }.must_raise ArgumentError
        end
      end
    end

    describe 'a subject' do
      let(:cgn_document) { 'f<self>_&_' }
      let(:ruby_object) do
        {
          :"...ally?" => false,
          actor: :self,
          state: {
            :"...last_moved_actor?" => nil,
            :"...previous_moves_counter" => nil
          }
        }
      end

      it 'parses a CGN string' do
        subject.load(cgn_document).hash.must_equal ruby_object.hash
      end

      describe 'errors' do
        it 'raises without a well-formed subject' do
          -> { subject.load 'f<foo>_&_' }.must_raise ArgumentError
        end
      end
    end
  end

  describe '.dump' do
    describe 'a relationship' do
      let(:cgn_document) { 'an_ally_actor' }
      let(:ruby_object) { :an_ally_actor }

      it 'emits some CGN' do
        subject.dump(ruby_object).must_equal cgn_document
      end

      describe 'errors' do
        it 'raises without a relationship' do
          -> { subject.dump 'foobar' }.must_raise ArgumentError
        end
      end
    end

    describe 'a boolean' do
      let(:cgn_document) { 'f' }
      let(:ruby_object) { false }

      it 'emits some CGN' do
        subject.dump(ruby_object).must_equal cgn_document
      end

      describe 'errors' do
        it 'raises without a boolean' do
          -> { subject.dump 'foobar' }.must_raise ArgumentError
        end
      end
    end

    describe 'null' do
      let(:cgn_document) { '_' }
      let(:ruby_object) { nil }

      it 'emits some CGN' do
        subject.dump(ruby_object).must_equal cgn_document
      end

      describe 'errors' do
        it 'raises without null' do
          -> { subject.dump '' }.must_raise ArgumentError
        end
      end
    end

    describe 'a subject' do
      let(:cgn_document) { 'f<self>_&_' }
      let(:ruby_object) do
        {
          :"...ally?" => false,
          actor: :self,
          state: {
            :"...last_moved_actor?" => nil,
            :"...previous_moves_counter" => nil
          }
        }
      end

      it 'emits some CGN' do
        subject.dump(ruby_object).must_equal cgn_document
      end

      describe 'errors' do
        it 'raises without a well-formed subject' do
          -> { subject.dump 'f<foo>_&_' }.must_raise ArgumentError
        end
      end
    end
  end
end
