require_relative '_test_helper'

describe Sashite::CGN::Pattern do
  subject { Sashite::CGN::Pattern }

  let(:cgn_document) do
    't<self>_&_^shift[-1,0]_=_@f+all~_@f+all%self; ' +
    't<self>_&_^remove[-1,0]1=_@f+all~_@an_enemy_actor+all%self'
  end

  let(:ruby_object) do
    [
      {
        :"subject" => {
          :"...ally?" => true,
          :"actor" => :self,
          :"state" => {
            :"...last_moved_actor?" => nil,
            :"...previous_moves_counter" => nil
          }
        },
   
        :"verb" => {
          :"name" => :shift,
          :"vector" => {:"...maximum_magnitude" => nil, :direction => [-1,0]}
        },
   
        :"object" => {
          :"src_square" => {
            :"...attacked?" => nil,
            :"...occupied!" => false,
            :"area" => :all
          },
          :"dst_square" => {
            :"...attacked?" => nil,
            :"...occupied!" => false,
            :"area" => :all
          },
          :"promotable_into_actors" => [:self]
        }
      },
      {
        :"subject" => {
          :"...ally?" => true,
          :"actor" => :self,
          :"state" => {
            :"...last_moved_actor?" => nil,
            :"...previous_moves_counter" => nil
          }
        },
   
        :"verb" => {
          :"name" => :remove,
          :"vector" => {:"...maximum_magnitude" => 1, :direction => [-1,0]}
        },
   
        :"object" => {
          :"src_square" => {
            :"...attacked?" => nil,
            :"...occupied!" => false,
            :"area" => :all
          },
          :"dst_square" => {
            :"...attacked?" => nil,
            :"...occupied!" => :an_enemy_actor,
            :"area" => :all
          },
          :"promotable_into_actors" => [:self]
        }
      }
    ]
  end

  describe '.load' do
    it 'parses a CGN string' do
      subject.load(cgn_document).hash.must_equal ruby_object.hash
    end

    describe 'errors' do
      it 'raises with several identical patterns' do
        -> { subject.load 't<self>_&_^shift[-1,0]_=_@f+all~_@f+all%self; ' +
                          't<self>_&_^shift[-1,0]_=_@f+all~_@f+all%self' }.must_raise ArgumentError
      end
    end
  end

  describe '.dump' do
    it 'emits some CGN' do
      subject.dump(ruby_object).must_equal cgn_document
    end

    describe 'errors' do
      it 'raises with several identical patterns' do
        -> { subject.dump 't<self>_&_^shift[-1,0]_=_@f+all~_@f+all%self; ' +
                          't<self>_&_^shift[-1,0]_=_@f+all~_@f+all%self' }.must_raise ArgumentError
      end
    end
  end
end
