require_relative '_test_helper'

describe Sashite::CGN::PreviousMovesCounter do
  subject { Sashite::CGN::PreviousMovesCounter }

  describe '.load' do
    describe 'null' do
      let(:cgn_document) { '_' }
      let(:ruby_object) { nil }

      it 'parses a CGN string' do
        subject.load(cgn_document).must_equal ruby_object
      end

      describe 'errors' do
        it 'raises without null' do
          -> { subject.load '' }.must_raise ArgumentError
        end
      end
    end

    describe 'an unsigned integer' do
      let(:cgn_document) { '0' }
      let(:ruby_object) { 0 }

      it 'parses a CGN string' do
        subject.load(cgn_document).must_equal ruby_object
      end

      describe 'errors' do
        it 'raises with a negative integer' do
          -> { subject.load '-4' }.must_raise ArgumentError
        end
      end
    end
  end

  describe '.dump' do
    describe 'null' do
      let(:cgn_document) { '_' }
      let(:ruby_object) { nil }

      it 'emits some CGN' do
        subject.dump(ruby_object).must_equal cgn_document
      end

      describe 'errors' do
        it 'raises without null' do
          -> { subject.dump '' }.must_raise ArgumentError
        end
      end
    end

    describe 'an unsigned integer' do
      let(:cgn_document) { '0' }
      let(:ruby_object) { 0 }

      it 'emits some CGN' do
        subject.dump(ruby_object).must_equal cgn_document
      end

      describe 'errors' do
        it 'raises with a negative integer' do
          -> { subject.dump '-4' }.must_raise ArgumentError
        end
      end
    end
  end
end
