require_relative '_test_helper'

describe Sashite::CGN::UnsignedInteger do
  subject { Sashite::CGN::UnsignedInteger }

  describe '.load' do
    describe 'a zero' do
      let(:cgn_document) { '0' }
      let(:ruby_object) { 0 }

      it 'parses a CGN string' do
        subject.load(cgn_document).must_equal ruby_object
      end

      describe 'errors' do
        it 'raises with a double zero' do
          -> { subject.load '00' }.must_raise ArgumentError
        end
      end
    end

    describe 'an unsigned integer excluding zero' do
      let(:cgn_document) { '42' }
      let(:ruby_object) { 42 }

      it 'parses a CGN string' do
        subject.load(cgn_document).must_equal ruby_object
      end

      describe 'errors' do
        it 'raises with a negative integer' do
          -> { subject.load '-4' }.must_raise ArgumentError
        end
      end
    end
  end

  describe '.dump' do
    describe 'a zero' do
      let(:cgn_document) { '0' }
      let(:ruby_object) { 0 }

      it 'emits some CGN' do
        subject.dump(ruby_object).must_equal cgn_document
      end

      describe 'errors' do
        it 'raises with a double zero' do
          -> { subject.dump '00' }.must_raise ArgumentError
        end
      end
    end

    describe 'an unsigned integer excluding zero' do
      let(:cgn_document) { '42' }
      let(:ruby_object) { 42 }

      it 'emits some CGN' do
        subject.dump(ruby_object).must_equal cgn_document
      end

      describe 'errors' do
        it 'raises with a negative integer' do
          -> { subject.dump '-4' }.must_raise ArgumentError
        end
      end
    end
  end
end
